package handlers

import (
	"bytes"
	"fmt"
	"go/ast"
	"go/format"
	"go/parser"
	"go/token"
	"log"
	"os"
	"path/filepath"
	"regexp"
	"runtime"
	"strings"
	"testing"
	"text/template"
	"unicode"
)

func TestGenActionsHandlerFunc(t *testing.T) {
	createActionRoutesGen()
}

// 生成路由代码
func createActionRoutesGen() {
	var err error
	moduleName := GetModuleName(6)
	af := getActionsInfo(moduleName)
	tpl, err := template.ParseGlob("./*.tpl")
	if err != nil {
		panic(err)
	}

	// genFile(tpl, "gen_action_routes.tpl", "./action_routes_gen.go", af)
	genFile(tpl, "types_gen.tpl", "./actions/types_gen.go", map[string]any{
		"ActionFunctions": af,
		"ModuleName":      moduleName,
	})
}

func genFile(tpl *template.Template, executeTemplate string, genFilePath string, tplParam any) {

	var buf bytes.Buffer
	err := tpl.ExecuteTemplate(&buf, executeTemplate, tplParam)
	if err != nil {
		panic(err)
	}

	data, err := format.Source(buf.Bytes())
	if err != nil {
		panic(err)
	}

	// log.Println(string(data))

	f, err := os.OpenFile(genFilePath, os.O_TRUNC|os.O_WRONLY|os.O_CREATE, 0644)
	if err != nil {
		panic(err)
	}

	f.Write(data)
	f.Close()
}

type actionsFunc struct {
	ModuleName  string
	ActionName  string
	FuncName    string
	ParamStruct *paramStruct
}

type paramStructField struct {
	ParamName      string
	ParamNameCamel string
	ParamType      string
	ParamBinding   string
}

type paramStruct struct {
	ParamStructName string
	ParamFields     []paramStructField
}

func getActionsInfo(moduleName string) map[string]*actionsFunc {

	regActionRe := regexp.MustCompile(`(?i)action +([^ ]+)`)
	paramStructRe := regexp.MustCompile(` +([a-zA-Z_\-/]+)(\?{0,1}): +(\w+) ?;`)

	dir := "actions"
	actionsMap := make(map[string]*actionsFunc)
	// actionsMap["ModuleName"] = moduleName
	err := filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
		if err != nil {
			return err
		}

		if !info.IsDir() && strings.HasSuffix(info.Name(), ".go") {
			fset := token.NewFileSet()

			file, err := parser.ParseFile(fset, path, nil, parser.ParseComments)
			if err != nil {
				return err
			}

			for _, decl := range file.Decls {
				fn, ok := decl.(*ast.FuncDecl)
				if !ok {
					continue
				}

				if fn.Doc != nil {
					var pstru *paramStruct
					var af *actionsFunc
					for _, comment := range fn.Doc.List {
						// 匹配 @Action 提取
						if strings.Contains(comment.Text, "@Action") || strings.Contains(comment.Text, "@action") {
							matches := regActionRe.FindStringSubmatch(comment.Text)
							if len(matches) != 0 {
								af = &actionsFunc{
									ModuleName: moduleName,
									ActionName: matches[1],
									FuncName:   fn.Name.Name,
								}
								actionsMap[af.ActionName] = af
							} else {
								log.Panicf("action 无法匹配: %s", comment.Text)
							}
							continue
						}

						psvalue := paramStructRe.FindAllStringSubmatch(comment.Text, -1)
						if len(psvalue) != 0 {

							if pstru == nil {
								pstru = &paramStruct{
									ParamStructName: fn.Name.Name + "Param",
								}
								af.ParamStruct = pstru
							}

							pf := paramStructField{}

							paramInfo := psvalue[0]

							pf.ParamName = paramInfo[1]
							pf.ParamNameCamel = toCamelCase(pf.ParamName)

							paramRequired := paramInfo[2]
							if paramRequired != "" {
								pf.ParamBinding = "required"
							} else {
								pf.ParamBinding = "-"
							}

							pf.ParamType = paramInfo[3]

							pstru.ParamFields = append(pstru.ParamFields, pf)
						}
					}
				}
			}
		}

		return nil
	})

	if err != nil {
		panic(fmt.Sprint("Error:", err))
		// return actionsMap
	}

	for action, funcNames := range actionsMap {
		fmt.Printf("Action: %s %v\n", action, funcNames)
	}

	return actionsMap
}

func toCamelCase(s string) string {
	var sb strings.Builder
	capitalizeNext := true
	for _, r := range s {
		if r == '_' || r == '/' || r == '-' {
			capitalizeNext = true
		} else if capitalizeNext {
			sb.WriteRune(unicode.ToUpper(r))
			capitalizeNext = false
		} else {
			sb.WriteRune(unicode.ToLower(r))
		}
	}
	return sb.String()
}

func GetModuleName(maxDepth int) string {
	gomodPath, err := FindGoModFile(maxDepth, "go.mod")
	if err != nil {
		panic(err)
	}
	// 解析当前目录
	data, err := os.ReadFile(gomodPath)
	if err != nil {
		panic(err)
	}
	matches := regexp.MustCompile(`module +([^ \n]+)`).FindStringSubmatch(string(data))
	if len(matches) == 0 {
		panic("can't find module name")
	}
	return matches[1]
}

func FindGoModFile(maxDepth int, fileName string) (string, error) {
	_, currentFile, _, ok := runtime.Caller(0)
	if !ok {
		return "", fmt.Errorf("failed to get current file path")
	}

	currentDir := filepath.Dir(currentFile)
	for i := 0; i < maxDepth; i++ {
		modFilePath := filepath.Join(currentDir, fileName)
		_, err := os.Stat(modFilePath)
		if err == nil {
			return modFilePath, nil
		} else if !os.IsNotExist(err) {
			return "", err
		}

		currentDir = filepath.Dir(currentDir)
	}

	return "", fmt.Errorf("go.mod file not found within %d parent directories", maxDepth)
}
